using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Media3D;

using NewtonDynamics;

namespace WPFPhysics1
{
	public class RectangleBody : Body
	{
		private double width;
		private double height;

		public RectangleBody( CCollision collision, double width, double height )
			: base( collision, null )
		{
			this.width = width;
			this.height = height;
			offset.X = -width / 2;
			offset.Y = -height / 2;
		}

		/// <summary>
		/// Create instance
		/// </summary>
		/// <param name="world">World</param>
		/// <param name="width">Width</param>
		/// <param name="height">Height</param>
		/// <returns>New instance</returns>
		public static RectangleBody Create( CWorld world, double mass, double width, double height, double depth )
		{
			CCollisionConvexPrimitives collision = new CCollisionConvexPrimitives( world );
			collision.CreateBox( new Vector3D( width, height, depth ), Matrix3D.Identity );

			RectangleBody body = ( mass < double.PositiveInfinity )
				? new RectangleBody( collision, width, height )
				: new StaticRectangleBody( collision, width, height );

			InertialMatrix inertialMatrix = collision.CalculateInertialMatrix;

			body.MassMatrix = new MassMatrix( (float)mass, inertialMatrix.m_Inertia * mass );
			body.CentreOfMass = inertialMatrix.m_Origin;

			collision.Release();

			return body;
		}

		public double Width { get { return width; } }
		public double Height { get { return height; } }
	}
}